#include "Mix3Plugin.h"
#include "Mix3FilterEARLY.h"
#include "Mix3FilterLATE.h"
#include "PluginHub.h"
#include "PluginDependency.h"
#include "PluginOptionList.h"
#include "PluginImageSettings.h"
#include "cmath"
#include "../PluginTools/ToolData.h"

#include <QDebug>
#include <QMessageBox>
#include <QPushButton>

extern "C" BIBBLE_API BaseB5Plugin *b5plugin() { return new Mix3Plugin; }

bool Mix3Plugin::init(PluginHub *hub, int id, int groupId, const QString &)
{
	if(hub == NULL) return false;
	
	m_hub = hub;
	m_id = id;
	m_groupId = groupId;
	return true;
}

// Registering the filters
bool Mix3Plugin::registerFilters()
{
        //Mix3Filter is added after Negative Vignette in the processing pipe
        Mix3FilterEARLY *Mix3EARLY = new Mix3FilterEARLY(m_hub, m_groupId);
        Mix3FilterLATE *Mix3LATE = new Mix3FilterLATE(m_hub, m_groupId);
        if ( (m_hub->addFilter(Mix3EARLY, PluginHub::Before, QString("RgbToLab"))) && (m_hub->addFilter(Mix3LATE, PluginHub::Before, QString("Curves"))) ) {
		qDebug() << "Successfully registered "<<Mix3EARLY->name()<<" before RgbToLab and "<<Mix3LATE->name()<<" before Curves using priority " <<priority();
        return true;
	}
        qDebug() << "Failed to load the Mix3 filters";
	return false;
}

//	Registering the options. It does the same thing as the B5 version, only the functions have different syntax
//	If you want translateable strings, add the tr(...) around it.
bool Mix3Plugin::registerOptions()
{
 
        bool b = true;
		b &= m_hub->addBoolOption( ID_MIX3_ENABLED,FP_MIX3_ENABLED,tr("Enable"),tr("Enable plugin"), FILTERNAME, false, false, 0);
		b &= m_hub->addBoolOption( ID_MIX3_EARLY,FP_MIX3_EARLY,tr("Early Mode"),tr("Run Mix3 just before LAB rather than before Curves"), FILTERNAME, false, false, 0);
		b &= m_hub->addIntOption( ID_MIX3_EXPOSURE,FP_MIX3_EXPOSURE,tr("Exposure"),tr("Adjust overall exposure"), FILTERNAME, 0, 0, 0);
        b &= m_hub->addIntOption( ID_MIX3_RED,FP_MIX3_RED,tr("Red"),tr("Adjust the contribution of the red channel"), FILTERNAME, 0, 0, 0);
		b &= m_hub->addIntOption( ID_MIX3_GREEN,FP_MIX3_GREEN,tr("Green"),tr("Adjust the contribution of the green channel"), FILTERNAME, 0, 0, 0);
		b &= m_hub->addIntOption( ID_MIX3_BLUE,FP_MIX3_BLUE,tr("Blue"),tr("Adjust the contribution of the blue channel"), FILTERNAME, 0, 0, 0);

        if ( b == false ) qDebug() << "Mix3Plugin: Unable to register options.";

        return b;
}

//-----------------------------------------------------------------------------
//	Handle settings change: if a control depends on an other (e.g. changing something enables the plugin)
void Mix3Plugin::handleSettingsChange(const PluginImageSettings &options, const PluginImageSettings &changed, int currentLayer)
{

    // Update our controls when settings are changed on the main layer
	if ((options.options(0) == NULL) || (changed.options(0) == NULL) || (currentLayer != 0))
		return;					// Only care about the main layer

}
//	Handle control change: if a control depends on an other (e.g. changing something enables the plugin)
void Mix3Plugin::handleControlChange( const QString &optionName, int groupId, int layer, const PluginImageSettings &options, const PluginOptionList &current, PluginOptionList &changes )
{

    //	some changes do not have options - check for NULL
    if ( options.options( layer ) == NULL )return;
    
    //	ignore changes that aren't for us
    if ( groupId != m_groupId ) return;
    
    bool bOk;
	    // if the plugin isn't enabled and a control is changed, enable the plugin
    if(!current.getBool(ID_MIX3_ENABLED,m_groupId,bOk)){

		if((optionName==FP_MIX3_EARLY)||(optionName==FP_MIX3_EXPOSURE)||(optionName==FP_MIX3_RED)||(optionName==FP_MIX3_GREEN)||(optionName==FP_MIX3_BLUE))
		{
			changes.setBool(ID_MIX3_ENABLED,m_groupId,true);
		}
    }
}

//  Handle hotness change:  when we arrive on a new image
void Mix3Plugin::handleHotnessChanged(const PluginImageSettings &options)
{
	Q_UNUSED(options);

}
//	Handle importB5
//	About button
void Mix3Plugin::showAbout() const
{

   QMessageBox::about( m_pWindow, tr("About Mix3 - v1.0.1"), tr("A simple black and white channel mixer for AfterShot Pro\nVersion 1.0.1 - 19 July 2012\nWritten by Ferdinand Paris (F_P).\nBased on a sample plugin by Barna Keresztes.\n\nThis plugin produces B&W images by mixing the R,G,B channels.  It roughly preserves average luminosity (although 'roughly' is the operative term) by taking a weighted average of R,G,B with the slider values as weights.\n\nThis plugin also has its own exposure slider, which means if you happen to mix the channels outside the (0,65535) interval you can use the exposure slider to bring the resulting luminosity back inside the interval without it being clipped.  (Note:  if you set each of the R,G,B sliders to -100 then the image will go black, as you've set each of the channels to zero. Any other combination of the sliders is ok, but some extreme settings can produce odd effects.)\n\nThe default position of the plugin is to run late in the processing pipe just before curves (but before Bez - Late).  The 'Early' option runs the plugin before sharpening (before Bez - Early)." )) ;

}

//	Reset button
void Mix3Plugin::resetControls() const
{
   if (PluginOptionList *options = m_hub->beginSettingsChange("Reset all controls")) {
			options->setBool(ID_MIX3_ENABLED,m_groupId,false);
			options->setBool(ID_MIX3_EARLY,m_groupId,false);
			options->setInt(ID_MIX3_EXPOSURE,m_groupId,00);
			options->setInt(ID_MIX3_RED,m_groupId,0);
			options->setInt(ID_MIX3_GREEN,m_groupId,0);
			options->setInt(ID_MIX3_BLUE,m_groupId,0);

		m_hub->endSettingChange();
   }
} 

bool Mix3Plugin::finish()
{
	// Connect to settingsChanged so we can keep our custom controls in sync
	connect(m_hub, SIGNAL(settingsChanged(const PluginImageSettings &, const PluginImageSettings &, int)), this, SLOT(handleSettingsChange(const PluginImageSettings &, const PluginImageSettings &, int)));

    // Connect to controlChanged so we can do some settings dependency logic
    connect( m_hub, SIGNAL( controlChanged( const QString &, int, int , const PluginImageSettings &, const PluginOptionList &, PluginOptionList &) ), SLOT( handleControlChange( const QString &, int, int , const PluginImageSettings &, const PluginOptionList &, PluginOptionList & ) ) );

	// Connect to hotnessChanged so we can setup our custom controls when the active version changes
	connect(m_hub, SIGNAL(hotnessChanged(const PluginImageSettings &)), this, SLOT(handleHotnessChanged(const PluginImageSettings &)));

    return true;
}

PluginDependency *Mix3Plugin::createDependency(const QString &depName)
{
	// Q_UNUSED(name);
    if (depName == "ToolData") {
        ToolData *toolData = new ToolData();
        if (toolData) {
            toolData->owner = name();
            toolData->group = group();
            toolData->ownerId = m_id;
            toolData->groupId = m_groupId;
            toolData->addEnabledId(m_hub->optionIdForName("mix3_enabled",m_id), "On", "Enable Mix3", "This indicates whether Mix3 is enabled");
            qDebug() << "asPluginManager: createDependency ToolData" <<
            toolData;
        return toolData;
        }
    }
	return NULL;
}

QList<QString> Mix3Plugin::toolFiles()
{
	return QList<QString>();
}

QList<QWidget*> Mix3Plugin::toolWidgets()
{
	return QList<QWidget*>();
}

//	Connect the About button to the showAbout() function
void Mix3Plugin::toolWidgetCreated(QWidget *uiWidget)
{
	if (uiWidget == NULL) return;
	if ( QPushButton * pReset = uiWidget->findChild<QPushButton*>("mix3_reset_button"))
            connect( pReset, SIGNAL( clicked() ), SLOT( resetControls() ) );
    else qDebug() << "Mix3: Reset button problem.";
    if ( QPushButton * pAbout = uiWidget->findChild<QPushButton*>("mix3_about_button"))
        connect( pAbout, SIGNAL( clicked() ), SLOT( showAbout() ) );
    else qDebug() << "Mix3: About button problem.";
        m_pWindow = uiWidget->window();
}
